""" 
obsinfo information file routines, contained in superclass OBSMetadata for generality
"""
# Standard library modules
import json
import pprint
from pathlib import Path, PurePath 
import sys
import re
import warnings
warnings.simplefilter("once")
warnings.filterwarnings("ignore", category=DeprecationWarning)
from urllib.parse import urlparse
from urllib.parse import unquote

# Non-standard modules
import jsonschema
import jsonref
import yaml


# Local modules
from ..misc import yamlref
from ..misc.yamlref import JsonLoader
from ..misc.remoteGitLab import gitLabFile
from ..misc.discoveryfiles import Datapath

root_symbol = "#"
VALID_FORMATS = ["JSON", "YAML"]
VALID_TYPES = [
    "campaign",
    "network",
    "station",
    "instrumentation",
    "datalogger",
    "preamplifier",
    "sensor",
    "response",
    "stage",
    "filter",
]

class ObsMetadata(dict):
    
    def __init__(self, *args, **kwargs):
        """
        Constructor, simply create a dict
        """
        
        super().__init__(*args, **kwargs)
        #self.convert_to_obsmetadata()
        
    def convert_to_obsmetadata(self):
        """
        Make all contained dictionaries ObsMetadata
        """
        for key, value in self.items():
            if isinstance(value, dict):
                self[key] = self.__class__(value)
                self[key].convert_to_obsmetadata()
            elif isinstance(value, list):
                for x in value:
                   if isinstance(x, dict):
                      x = self.__class__(x)
                      x.convert_to_obsmetadata()    
        
    def list_valid_types():
        """
        Returns a list of valid information file types
        """
        return VALID_TYPES


    def get_information_file_type(filename):
        """
        Determines the type of a file, assuming that the filename is "*.{TYPE}.{SOMETHING}
        """
    
        #the_type = filename.split(".")[-2].split("/")[-1].lower()
        stem_file = PurePath(filename).stem
        the_type = (PurePath(stem_file).suffix)[1:]
        
        if the_type in VALID_TYPES:
            return the_type
        print(f"Unknown type: {the_type}")
        sys.exit(1)
    
    
    def validate(self, datapath, filename, format=None, type=None, verbose=False,
                 schema_file=None, quiet=False):
        """
        Validates a YAML or JSON file against schema
        type: "network", "datalogger", "preamplifier", "sensor", "response", "filter"
        format: "JSON" or "YAML"
        
        if type and/or format are not provided, tries to figure them out from the
        filename, which should be "*{TYPE}.{FORMAT}
        """
    
        if quiet:
            verbose = False
    
        if not type and not schema_file:
            type = ObsMetadata.get_information_file_type(filename)
    
        if verbose:
            print(f"instance = {filename}")
            
        dp = Datapath()
        
        instance = ObsMetadata.read_json_yaml_ref(filename, dp, format=format)
        # instance = read_json_yaml(filename, format=format)
        
        base_file = (type if not schema_file else schema_file) + ".schema.json"  
        schema_file = PurePath(datapath).joinpath(base_file)
        base_uri = unquote(PurePath(schema_file).as_uri())
       
        with open(schema_file, "r") as f:
            try:
                schema = yamlref.loads(f.read(), base_uri=base_uri, jsonschema=True, 
                                       datapath=datapath, recursive=True)
                
            except json.decoder.JSONDecodeError as e:
                print(f"JSONDecodeError: Error loading JSON schema file: {schema_file}")
                print(str(e))
                return False
            except:
                print(f"Error loading JSON schema file: {schema_file}")
                print(sys.exc_info()[1])
                return False
               
        # Lazily report all errors in the instance
        # ASSUMES SCHEMA IS DRAFT-04 (I couldn't get it to work otherwise)
    
        try:
            
            if verbose:
                print(f"schema =   {schema_file.name}")
                print("\tTesting schema ...", end="")
             
            # Check schema first    
            jsonschema.Draft4Validator.check_schema(schema)
            
            if verbose:
                print("OK")
                print("\tTesting instance ...", end="")
    
            v = jsonschema.Draft4Validator(schema)
    
            if not v.is_valid(instance):
                print("")
                for error in sorted(v.iter_errors(instance), key=str):
                    print("\t\t", end="")
                    for elem in error.path:
                        print(f"['{elem}']", end="")
                    print(f": {error.message}")
                print("\tFAILED")
                return False
            else:
                if verbose:
                    print("OK")
                return True
        except jsonschema.ValidationError as e:
            print("\t" + e.message)
            return False
    
    
    def get_information_file_format(filename):
        """
        Determines if the information file is in JSON or YAML format
        
        Assumes that the filename is "*.{FORMAT}
        """
        suffix = PurePath(filename).suffix
        format = suffix[1:].upper()
        if format in VALID_FORMATS:
            return format
        print(f'Unknown format: {format}')
        sys.exit(1)
        
    def get_information_file_type(filename):
        """
        Determines if the information file is in JSON or YAML format
        
        Assumes that the filename is "*.{FORMAT}
        """
        stem = PurePath(filename).stem
        suffix = PurePath(stem).suffix
        type = suffix[1:]
        if type in VALID_TYPES:
            return type
        print(f'Unknown format: {type}')
        sys.exit(1)
    
    
    def read_json_yaml(filename, format=None):
        """ Reads a JSON or YAML file.  Does NOT use jsonReference """
        if not format: #validate format is legal. Otherwise program will exit
            format = ObsMetadata.get_information_file_format(filename)
    
        with open(filename, "r") as f:
            if format == "YAML":
                try:
                    element = yaml.safe_load(f)
                except:
                    print(f"Error loading YAML file: {filename}")
                    print(sys.exc_info()[1])
                    return
            else:
                try:
                    element = json.load(f)
                except JSONDecodeError as e:
                    print(f"JSONDecodeError: Error loading JSON file: {filename}")
                    print(str(e))
                    return
                except:
                    print(f"Error loading JSON file: {filename}")
                    print(sys.exc_info()[1])
                    return
    
        return element
    
    def read_json_yaml_ref_datapath(filename, datapath, format=None):
        """ 
        Reads a JSON or YAML file using jsonReference using OBSINFO_DATAPATH
        """
        
        if not format:
            format = ObsMetadata.get_information_file_format(filename)
            
        bu = unquote(filename)
         
        if gitLabFile.isRemote(bu):
            base_uri = unquote(urlparse(bu).path)
            loader = JsonLoader()
            jsonstr = loader.get_remote_json(bu, base_uri=base_uri, datapath=datapath)
            return yamlref.loads(jsonstr, base_uri=base_uri, datapath=datapath)
        else:
            base_uri = Path(bu).as_uri()
            with open(unquote(filename), "r") as f:
                return yamlref.load(f, base_uri=base_uri, datapath=datapath)
            
    def read_json_yaml_ref(filename, datapath, format=None):
        """ 
        Reads a JSON or YAML file using jsonReference
        Like read_json_yaml_ref, but does not look for files in OBSINFO_DATAPATH 
        But $ref within the datafailes withoout absolute or relative path will be still looked for
        in OBSINFO_DATAPATH directories
        """
        if not format:
            format = ObsMetadata.get_information_file_format(filename)
    
        bu = unquote(filename)
        base_uri = Path(bu).as_uri()
        #base_uri = base_path.as_uri()
               
        with open(filename, "r") as f:
            return yamlref.load(f, base_uri=base_uri, datapath=datapath)
    

    
    @staticmethod
    def read_info_file(filename, datapath, remote=False, format=None):
        """
        Reads an information file, force_local tells whether to use absolute/relative path
        or OBSINFO_DATAPATH
        """

        dict = ObsMetadata.read_json_yaml_ref(filename, datapath, format) if not remote \
                  else ObsMetadata.read_json_yaml_ref_datapath(filename, datapath, format)
        # Create an ObsMedatada object, which is a dir with some added methods
        return ObsMetadata(dict)
                
    def _validate_script(argv=None):
        """
        Validate an obsinfo information file
    
        Validates a file named *.{TYPE}.json or *.{TYPE}.yaml against the 
        obsinfo schema.{TYPE}.json file.
    
        {TYPE} can be campaign, network, instrumentation, instrument_components or
        response
        """
        from argparse import ArgumentParser
    
        parser = ArgumentParser(prog="obsinfo-validate", description=__doc__)
        parser.add_argument("info_file", help="Information file")
        parser.add_argument(
            "-t", "--type", choices=VALID_TYPES, default=None,
            help="Forces information file type (overrides interpreting from filename)",
        )
        parser.add_argument(
            "-f", "--format", choices=VALID_FORMATS, default=None,
            help="Forces information file format (overrides interpreting from filename)",
        )
        parser.add_argument(
            "-s", "--schema", default=None,
            help="Schema file (overrides interpreting from filename)",
        )
        parser.add_argument(
            "-v", "--verbose", action="store_true", help="increase output verbosity"
        )
        args = parser.parse_args()
    
        validate(args.info_file, format=args.format, type=args.type,
                schema_file=args.schema, verbose=args.verbose)
        
    
    def get_configured_element(self, key, channel_modification={}, selected_configuration={}, default=None): 
        """
        First converts any possible dictionary into an ObsMetadata object to be able to use
        this method recursively
        Returns the value corresponding to key in the following way:
        a) If key present in channel_modification, return the corresponding value
        b) Else if key present in selected_configuration, return the corresponding value
        c) Else if not and key present in self, return the corresponding value (this means config overrides info_dict(
        d) Else if key not present in either, return default 
        """
        
        value = self.get(key, default) #If no original value, use default step (c)

        # Use selected_configuration if present, else use original_value (which may be default) (a) and (b)
        # if here is for validation only
        if isinstance(selected_configuration, (dict, ObsMetadata)):
            value = selected_configuration.get(key, value)
        # Use channel_modification if present, else use original_value (which may be default) (a) and (b)
        # Cchannel modifications override selected configurations which override self which overrides default
        # but ONLY at the lowest (leaf) level, so we check whethere the value is a dictionary. If so, it won't be
        # substituted. This does not happen with configurations as they are complete substitutions of things such as
        # response stages or sample rates    
        if isinstance(channel_modification, (dict, ObsMetadata)):
            val = channel_modification.get(key, value)
            # Do not assign value if this is a dictionary, so it's not a leaf in the hierarchy
            if not isinstance(val, (dict, ObsMetadata)):
               value = val 

        if isinstance(value, dict): #Convert to ObsMetadata any dictionaries
            value = ObsMetadata(value)
            
        return value   
    
    def validate_dates(dates):
        
        if dates == None or not isinstance(dates, list):
            return []
        
        return [ObsMetadata.validate_date(dt) for dt in dates]
       
    def validate_date(date):
        
        if date == None or date == 0: #This is sometimes the default, the epoch date, 1/1/1970
            return 0
         
        regexp_date_UTC = re.compile("^[0-9]{4}[\-\/][0-9]{1,2}[\-\/]{1,2}")
        regexp_date_and_time_UTC = re.compile("^[0-9]{4}[\-\/][0-9]{1,2}[\-\/]{1,2}T[0-2][0-9]:[0-6][0-9]:[0-2]{0,1}[0-9]{0,1}Z{0.1}\.{0,1]0*")
        if re.match(regexp_date_UTC, date) or re.match(regexp_date_and_time_UTC, date): # Year starts string, everything OK
            return date
        else: #Assume it's a regular date
            warnings.warn(f'Date {date} is not UTC format, assuming regular dd/mm/yyyy format')
            regexp_date_normal = re.compile("[\-\/]")
            date_elements = re.split(regexp_date_normal, date)
            if len(date_elements) != 3:
                raise ValueError("Unrecognizable date, must either be UTC or dd/mm/yyyy or dd/mm/yy. Dashes can be used as separators")
            if len(date_elements[2]) == 2:
                date_elements[2] = "20" + date_elements[2] # Assume 21st century
                warnings.warn(f'Date {date} is two digits, assuming 21st century')
    
            return date_elements [2] + "-" + date_elements [1] + "-" + date_elements [0]
            
            
            
            
            
            


import requests
import base64
import json

from ..misc import yamlref


def constructURL(user = "404",repo_name= "404",path_to_file= "404",url= "404"):
  url = url.replace("{user}",user)
  url = url.replace("{repo_name}",repo_name)
  url = url.replace("{path_to_file}",path_to_file)
  return url

json_url ='https://api.github.com/repos/larean/obsinfo/contents/obsinfo/_examples/Information_Files/campaign/BBOBS.INSU-IPGP.network.yaml?ref=v0.108'

#json_url = constructURL(user,repo_name,path_to_file,json_url) #forms the correct URL
response = requests.get(json_url) #get data from json file located at specified URL 

if response.status_code == requests.codes.ok:
    jsonResponse = response.json()  # the response is a JSON
    #the JSON is encoded in base 64, hence decode it
    content = base64.b64decode(jsonResponse['content'])
    #convert the byte stream to string
    jsonString = content.decode('utf-8')
    print(jsonString)
    finalJson = yamlref.loads(jsonString, base_uri = json_url)
else:
    print('Content was not found.')
    exit()

#for key, value in finalJson.items():
    #print("The key and value are ({}) = ({})".format(key, value))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Functions to print obsinfo objects

"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from future.builtins import *  # NOQA @UnusedWildImport

import os
from pathlib import Path, PurePath
import glob
import unittest
import inspect
import difflib
import re
from obspy.core.utcdatetime import UTCDateTime
# from pprint import pprint
# import xml.etree.ElementTree as ET
# from CompareXMLTree import XmlTree
from obsinfo.obsMetadata.obsmetadata import (ObsMetadata)
from ..instrumentation import (Instrumentation, InstrumentComponent,
                                     Datalogger, Preamplifier, Sensor,
                                     ResponseStages, Stage, Filter)
from ..instrumentation.instrumentation import (Location)
from ..network import (Station, Network)
from ..instrumentation.filter import (Filter, PolesZeros, FIR, Coefficients, ResponseList,
                     Analog, Digital, AD_Conversion)


class PrintObs(object):
    
    @staticmethod
    def print_component(obj, level="all"):
        if not obj:
            return
        print(obj)
        if level == "channel":
            return 
        
        print(obj.equipment)
        
    @staticmethod
    def print_instrumentation(obj, level="all"):
        print(obj)
        print("Instrument:\n")
        print(obj.equipment)
        for k in obj.channels:
            print(k)
            if level != "instrumentation":
                print(k.instrument)
                PrintObs.print_component(k.instrument.sensor, level)
                PrintObs.print_component(k.instrument.preamplifier, level)
                PrintObs.print_component(k.instrument.datalogger, level)
            
            if k.instrument.sensor.response_stages.stages and level != "component":
                for s in k.instrument.sensor.response_stages.stages:
                   print(s)
                   if level == "all":
                       print(s.filter) 
            
    
    @staticmethod            
    def print_station(obj, level="all"):
        print(obj)
        print(obj.locations)
        for k, v in obj.locations.items():
            print(f'Location {k}: {v}')
        if level != "station":
            PrintObs.print_instrumentation(obj.instrumentation, level)
    
    @staticmethod        
    def print_network(obj, level="all"):
        print("GUKI: {level}")
        print(obj)
        print(obj.operator)
        if level != "network":
            for v in obj.stations:
                PrintObs.print_station(v, level)
                   


"""
InstrumentComponent class and subclasses.

No StationXML equivalent.
"""
# Standard library modules
# import math as m
import warnings
warnings.simplefilter("once")
warnings.filterwarnings("ignore", category=DeprecationWarning)
import re
# Non-standard modules
from obspy.core.inventory.util import Equipment as obspy_Equipment
import obspy.core.util.obspy_types as obspy_types
from obspy.core.utcdatetime import UTCDateTime

from .response_stages import ResponseStages

from ..obsMetadata.obsmetadata import (ObsMetadata)

class InstrumentComponent(object):
    """
    InstrumentComponent superclass.  No obspy equivalent
    """
    def __init__(self, equipment, response_stages=[],
                 config_description=''):
        """
        Constructor
        """
        self.equipment = equipment
        self.config_description = config_description
             
        if not response_stages or not response_stages.stages:
            raise TypeError(f'No response stages in {type(self)}')
            
        self.response_stages = response_stages
        
        self.obspy_equipment = self.equipment.to_obspy() 
        
    def retrieve_configuration(comp_type, attributes_dict, configuration_selection=None):
        """
        Completes the component configuration.
        A configuration selector can be defined in the instrumentation class, otherwise a config default is used
        They may be used to select a configuration in the list of configuration definitions
        If a configuration matches the selector configuration its attributes will override or
        add to existing attributes.          
        """
        
        if not attributes_dict:
            return None
        
        configuration_default = attributes_dict.get('configuration_default', None)
        # If there is a selected configuration at the instrumentation level) use it, else use default
        # configuration at component level
        # If no default configuration selected_configuration will be None
        selected_configuration_key = configuration_selection \
                                    if configuration_selection else configuration_default

        if not selected_configuration_key:
            warnings.warn(f'No configuration key or default found in {comp_type}. \
            Configurations, if present, will not be applied')
            return {}
        
        all_configurations = attributes_dict.get('configuration_definitions', None)
        if not all_configurations:
            return {}
        
        selected_configuration = all_configurations.get(selected_configuration_key, {})
        
        if selected_configuration == {}:
            warnings.warn(f'Configuration selected "{selected_configuration_key}" \
             in {comp_type} not found in configuration definitions, using default configuration')
         
        return selected_configuration
            
    @staticmethod
    def dynamic_class_constructor(component_type, 
                                  attributes_dict, 
                                  channel_modif={},
                                  delay_correction=None, 
                                  config_selector=''):
        """
        Creates an appropriate Instrument_component subclass from an attributes_dict
        """
        if not attributes_dict.get(component_type, None):
            if component_type == 'preamplifier': #Only preamps are optional
               return None
            else:
                raise TypeError(f'No {component_type}')     
    
        selected_config = InstrumentComponent.retrieve_configuration(component_type, attributes_dict[component_type], 
                                                                     config_selector)
        
        if component_type == 'datalogger':
            obj = Datalogger.dynamic_class_constructor(ObsMetadata(attributes_dict['datalogger']), 
                                                       channel_modif.get('datalogger', {}),
                                                       selected_config, 
                                                       delay_correction)
        elif component_type == 'sensor':
            obj = Sensor.dynamic_class_constructor(ObsMetadata(attributes_dict['sensor']), 
                                                   channel_modif.get('sensor', {}),
                                                   selected_config, 
                                                   delay_correction) 
        elif component_type == 'preamplifier':
            obj = Preamplifier.dynamic_class_constructor(ObsMetadata(attributes_dict['preamplifier']), 
                                                         channel_modif.get('preamplifier', {}),
                                                         selected_config, 
                                                         delay_correction)  
        else:
            raise TypeError('Unknown InstrumentComponent ')
        
        if not obj.config_description:
            obj.config_description = config_selector if config_selector else ''
            
        obj.equipment.description += ('[config: ' + obj.config_description + ']') 
        
        return obj
    
    def __repr__(self):
        
        s = ''
        if self.equipment.description:
            s += f', description="{self.equipment.description}"'
        if self.response_stages:
            s += f'Response stages: {len(self.response_stages.stages)}'
            
        return s


class Datalogger(InstrumentComponent):
    """
    Datalogger Instrument Component. No obspy equivalent
    """
    def __init__(self, equipment, sample_rate, delay_correction=None, response_stages=[],
                 config_description=''):
        
        self.sample_rate = sample_rate
        self.delay_correction = delay_correction
                   
        super().__init__(equipment, response_stages, config_description)

    @classmethod
    def dynamic_class_constructor(cls, attributes_dict, channel_modif={}, selected_config={}, delay_correction=None):
        """
        Create Datalogger instance from an attributes_dict
        """
        if not attributes_dict:
            return None
        if not selected_config:
            selected_config = {} #A syntax error in the yaml file, two consecutive labels with no response stages, is avoided here
        
        # response_stages = reconfigure_component('response_stages', attributes_dict, selected_config, None)
        sample_rate = attributes_dict.get_configured_element('sample_rate', channel_modif, selected_config, None)
        delay_correction = attributes_dict.get_configured_element('delay_correction', channel_modif, selected_config, None)
        config_description = attributes_dict.get_configured_element('configuration_description', channel_modif, selected_config, '')
               
        #What the next line of code will do is to totally override response states in attribute_dict IF there is a selected_config with response_stages
        response_stages_list = attributes_dict.get_configured_element('response_stages', 
                                                                {}, 
                                                                selected_config,
                                                                 None)
        
        response_stages = ResponseStages(response_stages_list,
                                 channel_modif.get('response_modifications', {}), 
                                 selected_config.get('response_modifications', {}), 
                                 delay_correction)
                                  
        obj = cls(Equipment(ObsMetadata(attributes_dict.get('equipment', None)), channel_modif.get('equipment', {}), selected_config.get('equipment', {})),
                  sample_rate,
                  delay_correction,
                  response_stages,
                  config_description)
        
        return obj

    def __repr__(self):
        s = f'Datalogger(Sample Rate={self.sample_rate})'
        s += super().__repr__()
        
        return s


class Sensor(InstrumentComponent):
    """
    Sensor Instrument Component. No obspy equivalent
    """
    def __init__(self, equipment, seed_band_base_code, seed_instrument_code,
                 response_stages=[],
                 config_description=''):
        """
        Constructor

        :param equipment: Equipment information
        :type equipment: ~class `obsinfo.instrumnetation.Equipment`
        :param seed_band_base_code: SEED base code ("B" or "S") indicating
                                    instrument band.  Must be modified by
                                    obsinfo to correspond to output sample
                                    rate
        :type seed_band_base_code: str (len 1)
        :param seed_instrument code: SEED instrument code
        :type seed_instrument_code: str (len 1)
        :param seed_orientations: SEED orientation codes and corresponding
                                      azimuths and dips
        :type seed_orientations: dict

        """
       
        self.seed_band_base_code = seed_band_base_code
        self.seed_instrument_code = seed_instrument_code# dictionary
        
        super().__init__(equipment, response_stages, config_description)


    @classmethod
    def dynamic_class_constructor(cls, attributes_dict, channel_modif={}, selected_config={}, delay_correction=None):
        """
        Create Sensor instance from an attributes_dict
        """
        if not attributes_dict:
            return None
        if not selected_config:
            selected_config = {} #A syntax error in the yaml file, two consecutive labels with no response stages, is avoided here
        
        seed_dict = ObsMetadata(attributes_dict).get_configured_element('seed_codes', 
                                                                  channel_modif, 
                                                                  selected_config, {})
        
        #What the next line of code will do is to totally override response states in attribute_dict IF there is a selected_config with response_stages
        response_stages_list = attributes_dict.get_configured_element('response_stages', 
                                                                {}, 
                                                                selected_config,
                                                                 None)
        
        response_stages = ResponseStages(response_stages_list,
                                 channel_modif.get('response_modifications', {}), 
                                 selected_config.get('response_modifications', {}), 
                                 delay_correction)
                        
        obj = cls(Equipment(ObsMetadata(attributes_dict.get('equipment', None)), 
                            channel_modif.get('equipment', {}), 
                            selected_config.get('equipment', {})),
                            ObsMetadata(seed_dict).get_configured_element('band_base', 
                                                                          channel_modif, 
                                                                          selected_config, 
                                                                          None),
                            ObsMetadata(seed_dict).get_configured_element('instrument', 
                                                                          channel_modif, 
                                                                          selected_config,
                                                                          None),
                            response_stages,
                            attributes_dict.get_configured_element('configuration_description', 
                                                                   channel_modif, 
                                                                   selected_config, '')
                    )

        return obj

    def __repr__(self):
        
        s = ''
        
        s += f'\nSensor( "band code={self.seed_band_base_code}", "instrument code={self.seed_instrument_code}")'
        
        s += super().__repr__()
        
        return s


class Preamplifier(InstrumentComponent):
    """
    Preamplifier Instrument Component. No obspy equivalent
    """
    
    def __init__(self, equipment,  response_stages=[],
                 config_description=''):
        """
        Constructor

        :param equipment: Equipment information
        :type equipment: ~class `obsinfo.instrumnetation.Equipment`

        """
        
        super().__init__(equipment, response_stages, config_description)

    @classmethod    
    def dynamic_class_constructor(cls, attributes_dict, channel_modif={}, selected_config={}, delay_correction=None):
        """
        Create Preamplifier instance from an attributes_dict
        """
        if not attributes_dict:
            return None
        if not selected_config:
            selected_config = {} #A syntax error in the yaml file, two consecutive labels with no response stages, is avoided here
                
        #What the next line of code will do is to totally override response states in attribute_dict IF there is a selected_config with response_stages
        response_stages_list = attributes_dict.get_configured_element('response_stages', 
                                                                {}, 
                                                                selected_config, 
                                                                None)
        
        response_stages = ResponseStages(response_stages_list,
                                 channel_modif.get('response_modifications', {}), 
                                 selected_config.get('response_modifications', {}), 
                                 delay_correction)    
               
        obj = cls(Equipment(ObsMetadata(attributes_dict.get('equipment', None)), 
                            channel_modif.get('equipment', {}), 
                            selected_config.get('equipment', {})),
                            response_stages,
                            attributes_dict.get_configured_element('configuration_description', 
                                                                   channel_modif, 
                                                                   selected_config, 
                                                                   '')
                  )
        
        return obj
        
        
    def __repr__(self):
        s = f'Preamplifier()'
        s += super().__repr__()
        
        return s


class Equipment(obspy_Equipment):
    """
    Equipment class.

    Equivalent to obspy.core.inventory.util.Equipment
    """
    def __init__(self, attributes_dict, channel_modif={}, selected_config={}):
    
        self.type = attributes_dict.get_configured_element('type', 
                                                     channel_modif, 
                                                     selected_config, 
                                                     None)
        self.description = attributes_dict.get_configured_element('description', 
                                                            channel_modif, 
                                                            selected_config,
                                                             None)
        self.manufacturer = attributes_dict.get_configured_element('manufacturer', 
                                                             channel_modif, 
                                                             selected_config,
                                                              None)
        self.model = attributes_dict.get_configured_element('model', 
                                                      channel_modif, 
                                                      selected_config, 
                                                      None)
        self.vendor = attributes_dict.get_configured_element('vendor', 
                                                       channel_modif, 
                                                       selected_config, 
                                                       None)
        self.serial_number = attributes_dict.get_configured_element('serial_number', 
                                                              channel_modif, 
                                                              selected_config,
                                                              None)
        self.installation_date = ObsMetadata.validate_date(attributes_dict.get_configured_element('installation_date', 
                                                                  channel_modif, 
                                                                  selected_config,
                                                                  None)) 
        self.removal_date = ObsMetadata.validate_date(attributes_dict.get_configured_element('removal_date', 
                                                             channel_modif, 
                                                             selected_config, 
                                                             None))
        calibration_dates = ObsMetadata.validate_dates(attributes_dict.get_configured_element('calibration_dates', 
                                                                  channel_modif, 
                                                                  selected_config, 
                                                                  [])) 
        
        self.calibration_dates = calibration_dates
        self.resource_id = None
        
        equip = self.to_obspy()
        self.obspy_equipment = equip[0] if isinstance(equip, tuple) else equip
            
    
    def __repr__(self):
        s = f'Equipment('
        if self.type:
            s += f', Type={self.type}'
        if self.description:    
            s += f', Description={self.description}'
        if self.manufacturer:
            s += f', Manufacturer={self.manufacturer}'
        if self.model:
            s += f', Model={self.model}'
        if self.vendor:
            s += f', Vendor={self.vendor}'
        if self.serial_number:
            s += f', Serial Number={self.serial_number}'
        if self.installation_date:
            s += f', Installation Date={self.installation_date}'
        if self.removal_date:
            s += f', Removal Date={self.removal_date}'
        if self.calibration_dates:
            s += f', Calibration Date={self.calibration_dates}'
        s += ')'
        
        return s
    
    def to_obspy(self):
        """
        Convert an equipment (including the equipment description in components) to its obspy object
        """
        
        resource_id = None
        
        installation_date = UTCDateTime(self.installation_date if self.installation_date else 0)
        removal_date = UTCDateTime(self.removal_date if self.removal_date else 0)
        
        if isinstance(self.calibration_dates, list) and len(self.calibration_dates) > 0:
            calibration_dates = [UTCDateTime(dt) for dt in self.calibration_dates]
        else:
            calibration_dates = []
        #else:
        #    last_calibration_date = UTCDateTime(0) #Use last calibration date
        
        equip = obspy_Equipment(self.type, self.description, self.manufacturer, self.vendor,
                self.model, self.serial_number, installation_date, removal_date, 
                calibration_dates, resource_id)
        
        return equip        
         

    



import sys
import os
import re
from pathlib import Path, PurePath
import obsinfo

def main():
    file = Path(obsinfo.__file__).parent.joinpath("version.py")
    
    version={}
    with open(file) as fp:
        exec(fp.read(),version)
    
    name="obsinfo"
    version=version['__version__']
    
    print(name + " v" + version)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Functions to validate the lcheapo functions
There are two types of validating functionalities.
a) If the file name includes "test--attributes", ignore

b) If the file name is "normal", it will simply run through to make sure there are no errors  
"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from future.builtins import *  # NOQA @UnusedWildImport

import os
from pathlib import Path, PurePath
import glob
import inspect
import difflib
import re
import sys
from json.decoder import JSONDecodeError

from obspy.core.utcdatetime import UTCDateTime
# from pprint import pprint
# import xml.etree.ElementTree as ET
# from CompareXMLTree import XmlTree
from obsinfo.obsMetadata.obsmetadata import (ObsMetadata)
from obsinfo.instrumentation import (Instrumentation, InstrumentComponent,
                                     Datalogger, Preamplifier, Sensor,
                                     ResponseStages, Stage, Filter)
from obsinfo.instrumentation.instrumentation import (Location)
from obsinfo.network import (Station, Network)
from obsinfo.instrumentation.filter import (Filter, PolesZeros, FIR, Coefficients, ResponseList,
                     Analog, Digital, AD_Conversion)
from obsinfo.misc.printobs import  (PrintObs)
from obsinfo.misc.discoveryfiles import Datapath
import obsinfo 
from obsinfo.obsMetadata.obsmetadata import ObsMetadata
from obsinfo.misc.discoveryfiles import Datapath

# Succesful treatment 
EXIT_SUCCESS = 0
# File could not be processed, generic reasons 
EXIT_FAILURE = 1

# Reference: https://www.freebsd.org/cgi/man.cgi?query=sysexits&manpath=FreeBSD+4.3-RELEASE

# bad CLI parameters 
EXIT_USAGE = 64
#bad file content (input file : Network metadata file) 
EXIT_DATAERR = 65
# file not found (input file : Network metadata file) 
EXIT_NOINPUT = 66 
#A service is unavailable (This can occur if a support program or file does not exist) 
EXIT_UNAVAILABLE = 69 

# (technical/internal error)
# An internal software error has been detected 
EXIT_SOFTWARE = 70

#can't create file (report file, output file)
EXIT_CANTCREAT = 73 
#An error occurred while doing I/O on some file */
EXIT_IOERR = 74 

#Something was found in an unconfigured or misconfigured state */
EXIT_CONFIG = 78 


class ValidateObsinfo():
    """
    Test suite for obsinfo operations.
    """
    def setUp(self, verbose=True, print_output=True, debug=False):
        
        dp = Datapath()
        self.datapath = dp
        self.datapath.infofiles_path = dp.datapath_list 
        self.datapath.validate_path = Path(obsinfo.__file__).parent.joinpath('data', 'schemas')
    
        self.verbose = verbose
        self.print_output = print_output
        self.debug = debug

    def assertTextFilesEqual(self, first, second, msg=None):
        with open(first) as f:
            str_a = f.read()
        with open(second) as f:
            str_b = f.read()

        if str_a != str_b:
            first_lines = str_a.splitlines(True)
            second_lines = str_b.splitlines(True)
            delta = difflib.unified_diff(
                first_lines, second_lines,
                fromfile=first, tofile=second)
            message = ''.join(delta)

            if msg:
                message += " : " + msg

            self.fail("Multi-line strings are unequal:\n" + message)

    def test_validate_json(self):
        """
        Test validation on a YAML file.

        The test file as an $ref to a file that doesn't exist, a field that
        is not specified in the the schema, and lacks a field required in
        the schema
        """
        test_file = PurePath(self.datapath.validate_path).joinpath( 'json_testschema.json')
        test_schema = PurePath(self.datapath.validate_path).joinpath(
                                   'json_testschema.schema.json')
        # self.assertFalse(validate(test_file, schema_file=test_schema,
        #                           quiet=True))

        # Run the code
        cmd = f'obsinfo-validate -s {test_schema} {test_file} > temp'
        os.system(cmd)

        # Compare text files
        self.assertTextFilesEqual(
            'temp',
            PurePath(self.datapath.validate_path).joinpath( 'json_testschema.out.txt')
            )
        os.remove('temp')
        
    def validate_all_filters(self):
        """
        Validate all information files in <component>/responses/_filters" subdirectory
        If you wish to test individual files, use test_filter(file) with file an absolute or 
        relative file name.
        """
        
        for dir in self.datapath.infofiles_path:
            files_in_validate_dir = Path(dir).joinpath(
                "*rs", # includes sensors, preamplifiers and dataloggers
                "responses",
                "filters/*.yaml")
            
            filelist = glob.glob(str(files_in_validate_dir))
            
            for file in filelist:
                self.validate_filter(file)
        
    def validate_filters_in_directory(self, dir):
        """
        Validate all information files in filter directory.
        """
        if dir.is_dir():
            for file in dir.iterdir():
                self.validate_filter(file)     

    def validate_filter(self, info_file):
        """
        Validate a filter file. 
        """
        print(f'In validate all: {info_file}')      
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "filter", self.verbose, "filter", False)
            
        if ret and self.verbose:
           print(f'Filter test for: {info_file}: PASSED')
        
    def validate_all_stages(self):
        """
        Validate all information 
        files in each responses subdirectory.
        """
    
        for dir in self.datapath.infofiles_path:
            files_in_validate_dir = Path(dir).joinpath(
                "*rs", # includes sensors, preamplifiers and dataloggers
                "responses/*.yaml"
                )
            
            filelist = glob.glob(str(files_in_validate_dir))
            
            for file in filelist:
                self.validate_stage(file)
        
    def validate_stages_in_directory(self, dir):
        """
        Validate all information files in validate responses directory.
        """               
        datalogger_dir_re = re.compile(".*/dataloggers")
        sensor_dir_re = re.compile(".*/sensors")
        exception_re = re.compile(".*/test-with")
        
        if re.match(datalogger_dir_re, str(dir)) or re.match(sensor_dir_re, str(dir)):
            for file in (Path(dir).joinpath("responses")).iterdir():
                if not file.is_dir() and re.match(exception_re, str(file)) == None: 
                    self.validate_stage(file)        
        
    def validate_stage(self, info_file):
        """
        Test reading and converting to obspy a stage file with FIR filter.
        """
        
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "stage", self.verbose, "stage", False)
                
        if ret and self.verbose:
            print(f'Stage validate for: {info_file}: PASSED')
    
            
        
    def validate_all_components(self):
        """
        Test all information files in each responses subdirectory.
        """
        
        components_list = [
            "sensors",
            "preamplifiers",
            "dataloggers"
            ]
        
        for dir in self.datapath.infofiles_path:
            for comp in components_list:
                files_in_validate_dir = Path(dir).joinpath(
                    comp,
                    "*.yaml"  # includes sensors, preamplifiers and dataloggers
                    )
                
                filelist = glob.glob(str(files_in_validate_dir))
                
                for file in filelist:
                    ret = ObsMetadata().validate(self.datapath.validate_path, file, "yaml", comp, self.verbose, comp[:-1], False)
        
                    if ret and self.verbose:   
                        print(f'{file}: PASSED')
                    
        
    def validate_components_by_type(self, filelist):
        """
        Validate all information files in validate responses directory.
        """               
        datalogger_dir_re = re.compile("dataloggers")
        preamplifier_dir_re = re.compile("preamplifiers")
        sensor_dir_re = re.compile("sensors")

        for file in filelist:
            if re.match(datalogger_dir_re, str(filelist)):
                self.validate_datalogger(file)
                      
            elif re.match(preamplifier_dir_re, str(dir)):
                self.datapath.validate_preamplifier(file)
                      
            elif re.match(sensor_dir_re, str(dir)):
                self.validate_sensor(file)
                      
                
    def validate_datalogger(self, info_file):
        """
        Validate reading datalogger instrument_compoents.
        """
        
        #OJO: no configuraton passed from above. No delay_correction either.
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "datalogger", self.verbose, "datalogger", False)
        
        if ret and self.verbose:   
           print(f'{info_file}: PASSED')
           
           
    def validate_sensor(self, info_file):
        """
        Validate reading sensor instrument_compoents.
        """
        
        #OJO: no configuraton passed from above. No delay_correction either.
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "sensor", self.verbose, "sensor", False)
        
        if ret and self.verbose:   
           print(f'{info_file}: PASSED')
           
        
    def validate_preamplifier(self, info_file):
        """
        Validate reading sensor instrument_compoents.
        """
        
        #OJO: no configuraton passed from above. No delay_correction either.
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "preamplifier", self.verbose, "preamplifier", False)
        
        if ret and self.verbose:   
           print(f'{info_file}: PASSED')
           
           
    def validate_all_instrumentations(self, level="all"):
        """
        Validate all information files in each responses subdirectory.
        """
        
        for dir in self.datapath.infofiles_path:
            files_in_validate_dir = Path(dir).joinpath(
                "instrumentation/*.yaml" # includes sensors, preamplifiers and dataloggers
                )
            
            filelist = glob.glob(str(files_in_validate_dir))
            
            for file in filelist:
                self.validate_instrumentation(file)
                     
         
    def validate_instrumentation(self, info_file, level="all"):
        """
        Validate instrumentation.
        """
        
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "instrumentation", self.verbose, "instrumentation", False)
     
        if ret and self.verbose:   
           print(f'{info_file}: PASSED')
        
                     
    def validate_station(self, info_file='TEST.station.yaml', level="all"):
        """
        Validate a station.
        """
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "station", self.verbose, "station", False)
                
        if ret and self.verbose:
           print(f'Station validate for: {info_file}: PASSED')
           
           
    def validate_all_networks(self, level="all"):
        """
        Validate all information files in each responses subdirectory.
        """
        
        for dir in self.datapath.infofiles_path:
            files_in_validate_dir = Path(dir).joinpath(
                "network/*.yaml" # includes sensors, preamplifiers and dataloggers
                )
            
            filelist = glob.glob(str(files_in_validate_dir))
            
            for file in filelist:
                self.validate_network(file)
                     
             
    def validate_network(self, info_file='BBOBS.INSU-IPGP.network.yaml', level="all"):
        """
        Validate reading a network.
        """
        
        ret = ObsMetadata().validate(self.datapath.validate_path, info_file, "yaml", "network", self.verbose, "network", False)
        
        if ret and self.verbose:
           print(f'Network validate for: {info_file}: PASSED')
           
def main():

    verbose, print_output, debug, input_filename = retrieve_arguments()
    val = ValidateObsinfo()
    val.setUp(verbose, print_output) 
    dp = Datapath() 
        
    try:               
        
        type = ObsMetadata.get_information_file_type(input_filename)
         
        if verbose:
            print(f'Validating {type} file: {input_filename}')
            
        if type == "filter":
            val.validate_filter(val.datapath.build_datapath(input_filename))
        elif type == "stage":
            val.validate_stage(val.datapath.build_datapath(input_filename))
        elif type == "datalogger":
            val.validate_datalogger(val.datapath.build_datapath(input_filename))
        elif type == "preamplifier":
            val.validate_preamplifier(val.datapath.build_datapath(input_filename))
        elif type == "sensor":
            val.validate_sensor(val.datapath.build_datapath(input_filename))
        elif type == "instrumentation":
            val.validate_instrumentation(val.datapath.build_datapath(input_filename))
        elif type == "station":
            val.validate_station(val.datapath.build_datapath(input_filename))
        elif type == "network":
            val.validate_network(val.datapath.build_datapath(input_filename))
        
        """
        Need to check this code for relative paths
        if type == "filter":
            val.validate_filter(input_filename)
        elif type == "stage":
            val.validate_stage(input_filename)
        elif type == "datalogger":
            val.validate_datalogger(input_filename)
        elif type == "preamplifier":
            val.validate_preamplifier(input_filename)
        elif type == "sensor":
            val.validate_sensor(input_filename)
        elif type == "instrumentation":
            val.validate_instrumentation(input_filename)
        elif type == "station":
            val.validate_station(input_filename)
        elif type == "network":
            val.validate_network(input_filename)
        """
                       
    except TypeError:
        if debug:
            raise
        print("Illegal format: fields may be missing or with wrong format in input file.")
        sys.exit(EXIT_DATAERR)
    except (KeyError, IndexError):
        if debug:
            raise
        print("Illegal value in dictionary key or list index")
        sys.exit(EXIT_SOFTWARE)
    except ValueError:
        if debug:
            raise
        print("An illegal value was detected")
        sys.exit(EXIT_DATAERR)
    except FileNotFoundError:
        if debug:
            raise
        print("File could not be found")
        sys.exit(EXIT_NOINPUT)
    except JSONDecodeError:
        print("File and/or subfiles have an illegal format. Probably indentation or missing quotes/parentheses/brackets")
        if debug:
            raise 
        sys.exit(EXIT_DATAERR)
    except (IOError, OSError, LookupError):
        if debug:
            raise      
        print("File could not be opened or read")
        sys.exit(EXIT_UNAVAILABLE)
    except:
        sys.exit(EXIT_FAILURE)


def retrieve_arguments():
    
    options_dict = {
                       "quite" : "q",
                       "print_output" : "p",
                       "debug": "d",
                       "help" : "h",
                     }
    
    input_filename = output_filename = None
    verbose = True
    print_output = debug = False
    
    long_option = re.compile("^[\-][\-][a-zA-Z_]+$")
    short_option = re.compile("^[\-][a-zA-Z]$")
    possible_options = re.compile("^[qdph]+$")
    
    input_filename = ""
    
    option = None
    
    for arg in sys.argv[1:]:

        
        if re.match(long_option, arg):  
            option = options_dict.get(arg[2:])
        elif not arg[0] == "-":
            input_filename = arg
            continue  
        else:
            option = arg[1:]
        
        if not re.match(possible_options, option):
            s = f'Unrecognized option in command line: -{option}\n'
            s += usage()
            raise ValueError(s)
        
        for opt in option:
    
            if opt == "q":
                verbose = False
            elif opt == "p":
                print_output = True
            elif opt == "d":
                debug = True
            elif opt == "h": 
                print(usage())
                sys.exit()
             
    return (verbose, print_output, debug, input_filename)   

def usage():
    s = f'Usage: {sys.argv[0]} -vpdh <filename>\n'
    s += f'Where:\n'
    s += f'      -v or --verbose: prints test results\n'
    s += f'      -p or --print_output: prints a human readable version of processed information file\n'
    s += f'      -d or --debug: do not catch exceptions to show error trace.\n'
    s += f'      -h or --help: prints this message\n'
    
    return s
    
    
if __name__ == '__main__':
    main()


import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()
    print(long_description)
    
version={}
with open("obsinfo/version.py") as fp:
    exec(fp.read(),version)

setuptools.setup(
    name="obsinfo",
    version=version['__version__'],
    author="Wayne Crawford",
    author_email="crawford@ipgp.fr",
    description="Tools for documenting ocean bottom seismometer experiments and creating metadata",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://gitlab.com/resif/obsinfo/-/tree/v0.109/obsinfo",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=[
          'numpy>=1',
          'obspy>=1.1',
          'pyyaml>=3.0',
          'jsonschema>=2.6',
          'gitlab>=1.0.2',
          'jsonref>=0.2'
      ],
    entry_points={
        'console_scripts': [
            'obsinfo-local-validate=obsinfo.main.validate:main',
            'obsinfo-local-makeStationXML=obsinfo.main.makeStationXML:main',
            'obsinfo-print=obsinfo.main.print:print_obs',
            'obsinfo-setup=obsinfo.main.setupObsinfo:setup_obsinfo',
            'obsinfo-test=obsinfo.tests.test:run_suite_info_files',
            'obsinfo-print_version=obsinfo.print_version:main',
            'obsinfo-make_SDPCHAIN_scripts=obsinfo.addons.SDPCHAIN:_console_script',
            'obsinfo-make_LCHEAPO_scripts=obsinfo.addons.LCHEAPO:_console_script'
        ]
    },
    python_requires='>=3',
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    classifiers=(
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Physics"
    ),
    keywords='seismology OBS'
)



import sys
import os
import re
from pathlib import Path
from urllib import parse as urlparse
from urllib.parse import urljoin 
import warnings
warnings.simplefilter("once")
warnings.filterwarnings("ignore", category=DeprecationWarning)

from ..misc.remoteGitLab import gitLabFile

# We'll first create all the various objects. These strongly follow the
# hierarchy of StationXML files.

class Datapath(object):
    """
    Class to discover where information files are stored. GITLAB is used as a reserved word
    to signify the GITLAB repository
    """
    
    def __init__(self):
        """
        Create object
        """
        
        DATAPATH = os.environ.get('OBSINFO_DATAPATH')
        
        if DATAPATH:
           split_pattern = ':' if os.name == 'posix' else ';' 
           self.datapath_list = re.split(split_pattern, DATAPATH)
           i = 0
           for d in self.datapath_list:
               self.datapath_list[i] = re.sub('GITLAB/', 'http://www.gitlab.com/', d)
               i += 1
           
        else: 
           warnings.warn("OBSINFO_DATAPATH not set, defaulting to workind directory") 
           self.datapath_list = "./" # Use current directory as default.
        

    def build_datapath(self, file):
        """
        Create list of directories which may have data and schema
        """
        file, frag = urlparse.urldefrag(file)

        filestr = file if isinstance(file, str) else str(file)
        
        if not isinstance(file, Path):
            file = Path(file)
        
        if file.is_absolute(): 
            return filestr
    
        elif filestr[0:3] == "../" or filestr[0:2] == "./": # if path is absolute or relative to cwd:
            home = Path.cwd()
            self.datapath = str(Path.joinpath(home, file).resolve())
            return self.add_frag(self.datapath, frag)
                
        for dir in self.datapath_list:
             
            if gitLabFile.isRemote(dir):
                fn = urljoin(dir, filestr)
                if gitLabFile.get_gitlab_file(fn, False): #Check remote repository
                    self.datapath = fn
                    return self.add_frag(self.datapath, frag) # Don't forget to add frag back! 
            else:             
                fn = (Path(dir).resolve()).joinpath(file) 
                if os.path.isfile(str(fn)): # Check local repository
                   self.datapath = str(fn)
                   return self.add_frag(self.datapath, frag) # Don't forget to add frag back!

        raise FileNotFoundError(file)
    
    def add_frag(self, path, frag):
        
        return path + ("#" + frag if frag else "")
        


import os
import sys
import site
import platform
from pathlib import Path

def setup_obsinfo():
    
    version = platform.python_version()
         
    dest_dir = sys.argv[1] if len(sys.argv) > 1 else ""
    user_site = site.getusersitepackages() 
    path_to_package = Path(user_site).joinpath("obsinfo")
    setup_script = path_to_package.joinpath("bin", "setup-obsinfo")
    examples_dir = path_to_package.joinpath("_examples", "Information_Files")
    #Passes first argument as directory to move files to, from dir is 
    os.system("sh " +  str(setup_script) + " " + str(examples_dir) + " " + dest_dir)
    
    


import sys
import os
import re
import warnings
from pathlib import Path, PurePath

import obspy
from obspy.core.inventory import Inventory,  Station, Channel, Site
from obspy.core.inventory import Network as obspy_Network
from obspy.clients.nrl import NRL

from obsinfo.network import (Network)
from obsinfo.tests.test_all import(TestObsinfo)
from obsinfo.main.printobs import (PrintObs)
from obsinfo.obsMetadata.obsmetadata import (ObsMetadata)


# We'll first create all the various objects. These strongly follow the
# hierarchy of StationXML files.

class TestArgs(object):
    """
    Class to test how to start executing obsinfo in different modes
    """        
          
    def retrieve_arguments(arguments):
        
        options_dict = {
                           "output": "o",
                           "input" : "i",
                           "verbose" : "v",
                           "print_output" : "p",
                           "test" : "t",
                           "validate" : "d",
                         }
        
        input_filename = output_filename = None
        verbose = print_output = test = validate = False
        skip_next_arg = False
        input_found = False
        
        long_option = re.compile("^[\-][\-][a-zA-Z_]+$")
        short_option = re.compile("^[\-][a-zA-Z]$")
        possible_options = re.compile("^[vptdoih]+$")
        
        option = None
        
        for arg in arguments[1:]:

            if skip_next_arg:
                skip_next_arg = False
                continue
            
            if re.match(long_option, arg):  
                option = options_dict.get(arg[2:])
            elif not arg[0] == "-":
                    if not input_found:
                        input_filename = arg
                    else:
                        print(f'Warning: found two input files, one with option "-i" and one without. Retaining the first one and discarding "{arg}"')
                    break
            else:
                option = arg[1:]
            
            if not re.match(possible_options, option):
                s = f'Unrecognized option in command line: -{option}\n'
                s += TestArgs.usage()
                raise ValueError(s)
            
            for opt in option:
        
                if opt == "o":
                    if len(option) == 1:
                        output_filename = arguments[arguments.index("-o" if "-o" in arguments else "--output")+1]
                        skip_next_arg = True
                    else:
                        warnings.warn('-o option should stand alone and be followed by a filename')
                        break
                
                elif opt == "i":
                    if len(option) == 1:
                       input_filename = arguments[arguments.index("-i" if "-i" in arguments else "--input")+1]
                       input_found = True
                       skip_next_arg = True
                    else:
                        warnings.warn('-i option should stand alone and be followed by a filename')
                        break         
                
                elif opt == "v":
                    verbose = True
                elif opt == "p":
                    print_output = True 
                elif opt == "t":
                    test = True
                elif opt == "d":
                    validate = True
                elif opt == "h": 
                    print(TestArgs.usage())
                    sys.exit()
            
        return (verbose, print_output, test, validate, input_filename, output_filename)   
    
    def usage():
        s = f'Usage: {sys.argv[0]} -vptdh  -o <filename> [-i] <filename>\n'
        s += f'Where:\n'
        s += f'      -v or --verbose: prints processing progression\n'
        s += f'      -p or --print_output: prints a human readable version of processed information file\n'
        s += f'      -t or --test: enters test mode, produces no output\n'
        s += f'      -d or --validate: validates the YAML or JSON format of the information file with no further processing\n'
        s += f'      -h or --test: prints this message\n'
        s += f'      -o or --output: names the output file. Default is station.xml\n'
        s += f'      -i or --input: names the input file. The -i may be omitted and the argument will be understood as the input file name\n'
        s += f'      -i or --input: names the input file\n'
        
        return s


from .instrumentation import (Instrumentation, Instrument)
from .instrument_component import (InstrumentComponent, Datalogger, Sensor,
                                   Preamplifier, Equipment)
from .response_stages import (ResponseStages, Stage)
from .filter import Filter
